/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Maintained by Tom Wang <tom.wang@mustek.com.tw>

   Updates (C) 2001 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB flatbed scanners.  */

#include "mustek_usb_mid.h"
#include "mustek_usb_low.c"


/* adasensorcanon600 --------------------------------------------------------*/


#define GREENGRAY

/* Canon 600 Dpi */

static SANE_Word usb_mid_c600_s_wOpticalXDpi[] =
{ 600, 400, 300, 200, 150, 100, 50, 0 };

SANE_Bool
usb_mid_c600_PrepareRgbX600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P4P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 2);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 2);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P4P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 2);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 2);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareRgbX (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 50:
      usb_mid_c600_PrepareRgbX50Dpi (chip);
      break;
    case 100:
      usb_mid_c600_PrepareRgbX100Dpi (chip);
      break;
    case 150:
      usb_mid_c600_PrepareRgbX150Dpi (chip);
      break;
    case 200:
      usb_mid_c600_PrepareRgbX200Dpi (chip);
      break;
    case 300:
      usb_mid_c600_PrepareRgbX300Dpi (chip);
      break;
    case 400:
      usb_mid_c600_PrepareRgbX400Dpi (chip);
      break;
    case 600:
      usb_mid_c600_PrepareRgbX600Dpi (chip);
      break;
    default:
      DBG (6, "usb_mid_c600_PrepareRgbX switch (Dpi)\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_PrepareMonoX (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 50:
      usb_mid_c600_PrepareMonoX50Dpi (chip);
      break;
    case 100:
      usb_mid_c600_PrepareMonoX100Dpi (chip);
      break;
    case 150:
      usb_mid_c600_PrepareMonoX150Dpi (chip);
      break;
    case 200:
      usb_mid_c600_PrepareMonoX200Dpi (chip);
      break;
    case 300:
      usb_mid_c600_PrepareMonoX300Dpi (chip);
      break;
    case 400:
      usb_mid_c600_PrepareMonoX400Dpi (chip);
      break;
    case 600:
      usb_mid_c600_PrepareMonoX600Dpi (chip);
      break;
    default:
      DBG (6, "usb_mid_c600_PrepareMonoX switch (Dpi)\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c600_is600Mode (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c600_is600Mode: chip=%p, Dpi=%d\n", chip, Dpi);
  /* Cannon 600 CIS is 600 dpi mode always. */
  return SANE_TRUE;
}


/* adasensorcanon300600  -------------------------------------------------*/

/* for Canon 300/600 CIS */

static SANE_Word usb_mid_c300600_s_wOpticalXDpi[] =
  { 600, 400, 300, 200, 150, 100, 50, 0 };

SANE_Bool
usb_mid_c300600_PrepareRgbX600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 1);
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_TRUE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6);
  usb_low_SetSoftResample (chip, 1);
#ifdef GREENGRAY
  usb_low_SetLEDLightAll (chip, SANE_FALSE);
#else
  usb_low_SetLEDLightAll (chip, SANE_TRUE);
#endif
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareRgbX (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 50:
      usb_mid_c300600_PrepareRgbX50Dpi (chip);
      break;
    case 100:
      usb_mid_c300600_PrepareRgbX100Dpi (chip);
      break;
    case 150:
      usb_mid_c300600_PrepareRgbX150Dpi (chip);
      break;
    case 200:
      usb_mid_c300600_PrepareRgbX200Dpi (chip);
      break;
    case 300:
      usb_mid_c300600_PrepareRgbX300Dpi (chip);
      break;
    case 400:
      usb_mid_c300600_PrepareRgbX400Dpi (chip);
      break;
    case 600:
      usb_mid_c300600_PrepareRgbX600Dpi (chip);
      break;
    default:
      DBG (6, "usb_mid_c300600_PrepareRgbX switch (Dpi)");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_PrepareMonoX (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 50:
      usb_mid_c300600_PrepareMonoX50Dpi (chip);
      break;
    case 100:
      usb_mid_c300600_PrepareMonoX100Dpi (chip);
      break;
    case 150:
      usb_mid_c300600_PrepareMonoX150Dpi (chip);
      break;
    case 200:
      usb_mid_c300600_PrepareMonoX200Dpi (chip);
      break;
    case 300:
      usb_mid_c300600_PrepareMonoX300Dpi (chip);
      break;
    case 400:
      usb_mid_c300600_PrepareMonoX400Dpi (chip);
      break;
    case 600:
      usb_mid_c300600_PrepareMonoX600Dpi (chip);
      break;
    default:
      DBG (6, "usb_mid_c300600_PrepareMonoX switch (Dpi)\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_c300600_is600Mode (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c300600_is600Mode: chip=%p, Dpi=%d\n", chip, Dpi);
  switch (Dpi)
    {
    case 300:
    case 150:
    case 100:
    case 50:
      return SANE_FALSE;
    case 600:
    case 400:
    case 200:
      return SANE_TRUE;
    default:
      DBG (6, "usb_mid_c300600_is600Mode: wrong Dpi?\n");
      return SANE_FALSE;
      break;
    }
}


/* adamotor -------------------------------------------------------------*/


/* 1200 Dpi Motor */

static SANE_Word usb_mid_motor1200_s_wOpticalYDpi[] =
  { 1200, 600, 400, 300, 240, 200, 150, 120, 100, 75, 60, 50, 0 };

SANE_Bool
usb_mid_motor1200_PrepareRgbY1200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* Half */
  if (!usb_low_SetIO3 (chip, SANE_FALSE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_FALSE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY240Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 7))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_FALSE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_FALSE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_FALSE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_FALSE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY120Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 7))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_TRUE;
}

#if 0
SANE_Bool usb_mid_motor1200_PrepareRgbY300Dpi(CMA1017* chip)
{
	/* Setup Scan Here */
	if (!usb_low_SetCMTTable(chip,0,CH_GREEN,SANE_FALSE,SANE_TRUE)) return SANE_FALSE;
	if (!usb_low_SetCMTTable(chip,1,CH_BLUE,SANE_FALSE,SANE_TRUE)) return SANE_FALSE;
	if (!usb_low_SetCMTTable(chip,2,CH_RED,SANE_TRUE,SANE_TRUE)) return SANE_FALSE;
	if (!usb_low_SetCMTTable(chip,3,CH_GREEN,SANE_FALSE,SANE_FALSE)) return SANE_FALSE;
	if (!usb_low_SetCMTTable(chip,4,CH_GREEN,SANE_FALSE,SANE_FALSE)) return SANE_FALSE;
	if (!usb_low_SetCMTTableLength(chip,5)) return SANE_FALSE;
	if (!usb_low_SetCMTSecondPosition(chip,0)) return SANE_FALSE;
	if (!usb_low_SetCMTLoopCount(chip,0xefff)) return SANE_FALSE;
	if (!usb_low_SetMotorMovement(chip,SANE_TRUE,SANE_TRUE,SANE_TRUE)) return SANE_FALSE;/* BiFull */
	if (!usb_low_SetIO3(chip,SANE_FALSE)) return SANE_FALSE; /* (IO3) ? High power : Low power */
	return SANE_TRUE;
}
#endif

SANE_Bool
usb_mid_motor1200_PrepareRgbY150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY75Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY60Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 7))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 6))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY1200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 2))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY400Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY240Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 5))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY600Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 8, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 9, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 10, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 11, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 12, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 13, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 14, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 15, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 16, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 17, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 18, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 19, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 20, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 21, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 22, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 23, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 24, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 25, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 26, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 26))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 24))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY200Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

#if 0
SANE_Bool
usb_mid_motor1200_SetDummyLine (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 2))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_FALSE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}
#endif

SANE_Bool
usb_mid_motor1200_PrepareMonoY120Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 5))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_FALSE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 8, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 9, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 10, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 11, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 12, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 13, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 14, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 15, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 16, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 17, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 18, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 19, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 20, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 21, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 22, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 23, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 24, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 25, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 26, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 26))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 24))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY150Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 2))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY100Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 3))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY75Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY60Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 5))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY50Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 6))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbYHalf300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_RED, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 6))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbYBiFull300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_RED, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 6))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/* BiFull */
  if (!usb_low_SetIO3 (chip, SANE_FALSE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_RED, SANE_FALSE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 6))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;		/* BiFull x2 */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoYHalf300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 4))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;		/*Half */
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_TRUE;

}

SANE_Bool
usb_mid_motor1200_PrepareMonoYBiFull300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 2))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;

  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi (ma1017 * chip)
{
  /* Setup Scan Here */
  if (!usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE))
    return SANE_FALSE;
  if (!usb_low_SetCMTTableLength (chip, 2))
    return SANE_FALSE;
  if (!usb_low_SetCMTSecondPosition (chip, 0))
    return SANE_FALSE;
  if (!usb_low_SetCMTLoopCount (chip, 0xefff))
    return SANE_FALSE;
  if (!usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE))
    return SANE_FALSE;
  if (!usb_low_SetIO3 (chip, SANE_TRUE))
    return SANE_FALSE;		/* (IO3) ? High power : Low power */

  return SANE_TRUE;
}

SANE_Word
usb_mid_motor1200_RgbCapability (SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 1200:
    case 400:
    case 240:
      return 3008;		/* 2816 */; 
    case 600:
      return 3008;
    case 200:
    case 120:
      return 5056;
    case 300:
      return 3008;
    case 150:
      return 5056;
    case 100:
    case 75:
    case 60:
    case 50:
      return 10048;
    default:
      DBG (6, "CScanner__RgbCapability switch (Dpi)\n");
      return 0;
    }
}

SANE_Word
usb_mid_motor1200_MonoCapability (SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 1200:
    case 400:
    case 240:
      return 3008;
    case 600:
      return 3008;
    case 200:
    case 120:
      return 5056;
    case 300:
      return 5056;
    case 150:
    case 100:
    case 75:
    case 60:
    case 50:
      return 10048;
    default:
      DBG (3, "usb_mid_motor1200_MonoCapability: wrong Dpi?\n");
      return 0;
    }
}

SANE_Bool
usb_mid_motor1200_PrepareRgbY (ma1017 * chip, SANE_Word Dpi)
{
  usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE);
  /* No Motor & Forward */
  usb_low_SetMotorDirection (chip, SANE_FALSE);
  usb_low_EnableMotor (chip, SANE_TRUE);
  switch (Dpi)
    {
    case 1200:
      usb_mid_motor1200_PrepareRgbY1200Dpi (chip);
      break;
    case 400:
      usb_mid_motor1200_PrepareRgbY400Dpi (chip);
      break;
    case 240:
      usb_mid_motor1200_PrepareRgbY240Dpi (chip);
      break;
    case 600:
      usb_mid_motor1200_PrepareRgbY600Dpi (chip);
      break;
    case 200:
      usb_mid_motor1200_PrepareRgbY200Dpi (chip);
      break;
    case 120:
      usb_mid_motor1200_PrepareRgbY120Dpi (chip);
      break;
    case 300:
      usb_mid_motor1200_PrepareRgbY300Dpi (chip);
      break;
    case 150:
      usb_mid_motor1200_PrepareRgbY150Dpi (chip);
      break;
    case 100:
      usb_mid_motor1200_PrepareRgbY100Dpi (chip);
      break;
    case 75:
      usb_mid_motor1200_PrepareRgbY75Dpi (chip);
      break;
    case 60:
      usb_mid_motor1200_PrepareRgbY60Dpi (chip);
      break;
    case 50:
      usb_mid_motor1200_PrepareRgbY50Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareRgbY: wrong Dpi?\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareMonoY (ma1017 * chip, SANE_Word Dpi)
{
  usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE);
  /* No Motor & Forward */
  usb_low_SetMotorDirection (chip, SANE_FALSE);
  usb_low_EnableMotor (chip, SANE_TRUE);
  switch (Dpi)
    {
    case 1200:
      usb_mid_motor1200_PrepareMonoY1200Dpi (chip);
      break;
    case 400:
      usb_mid_motor1200_PrepareMonoY400Dpi (chip);
      break;
    case 240:
      usb_mid_motor1200_PrepareMonoY240Dpi (chip);
      break;
    case 600:
      usb_mid_motor1200_PrepareMonoY600Dpi (chip);
      break;
    case 200:
      usb_mid_motor1200_PrepareMonoY200Dpi (chip);
      break;
    case 120:
      usb_mid_motor1200_PrepareMonoY120Dpi (chip);
      break;
    case 300:
      usb_mid_motor1200_PrepareMonoY300Dpi (chip);
      break;
    case 150:
      usb_mid_motor1200_PrepareMonoY150Dpi (chip);
      break;
    case 100:
      usb_mid_motor1200_PrepareMonoY100Dpi (chip);
      break;
    case 75:
      usb_mid_motor1200_PrepareMonoY75Dpi (chip);
      break;
    case 60:
      usb_mid_motor1200_PrepareMonoY60Dpi (chip);
      break;
    case 50:
      usb_mid_motor1200_PrepareMonoY50Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareMonoY: wrong Dpi?\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareCalibrateRgbY (ma1017 * chip, SANE_Word Dpi)
{
  usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE);
  /* No Motor & Forward */
  usb_low_SetMotorDirection (chip, SANE_FALSE);
  usb_low_EnableMotor (chip, SANE_TRUE);
  switch (Dpi)
    {
    case 1200:
    case 400:
    case 240:
      usb_mid_motor1200_PrepareRgbYHalf300Dpi (chip);
      break;
    case 600:
    case 200:
    case 120:
      usb_mid_motor1200_PrepareRgbYBiFull300Dpi (chip);
      break;
    case 300:
      usb_mid_motor1200_PrepareRgbYHalf300Dpi (chip);
      break;
    case 150:
      usb_mid_motor1200_PrepareRgbYBiFull300Dpi (chip);
      break;
    case 100:
    case 75:
    case 60:
    case 50:
      usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareCalibrateRgbY: wrong Dpi?\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareCalibrateMonoY (ma1017 * chip, SANE_Word Dpi)
{
  usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE);
  /* No Motor & Forward */
  usb_low_SetMotorDirection (chip, SANE_FALSE);
  usb_low_EnableMotor (chip, SANE_TRUE);
  switch (Dpi)
    {
    case 1200:
    case 400:
    case 240:
      usb_mid_motor1200_PrepareMonoYHalf300Dpi (chip);
      break;
    case 600:
      usb_mid_motor1200_PrepareMonoYHalf300Dpi (chip);
      break;
    case 200:
    case 120:
      usb_mid_motor1200_PrepareMonoYBiFull300Dpi (chip);
      break;
    case 300:
      usb_mid_motor1200_PrepareMonoYBiFull300Dpi (chip);
      break;
    case 150:
    case 100:
    case 75:
    case 60:
    case 50:
      usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareCalibrateMonoY: wrong Dpi?\n");
      return SANE_FALSE;
      break;
    }
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareStep (ma1017 * chip, SANE_Word StepCount)
{
  usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE);
  /* Make it in 600dpi */
  usb_low_SetIO3 (chip, SANE_TRUE);	/* (IO3) ? High power : Low power */
  usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE);
  /* No Motor & Forward */
  if (StepCount == 1)
    {
      usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE);
      usb_low_SetCMTTableLength (chip, 1);
      usb_low_SetCMTSecondPosition (chip, 0);
      usb_low_SetCMTLoopCount (chip, StepCount);
    }
  else if (StepCount % 2 == 1)
    {
      usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE);
      usb_low_SetCMTTableLength (chip, 3);
      usb_low_SetCMTSecondPosition (chip, 1);
      usb_low_SetCMTLoopCount (chip, (StepCount - 1) / 2);
    }
  else
    {
      usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE);
      usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE);
      usb_low_SetCMTTableLength (chip, 2);
      usb_low_SetCMTSecondPosition (chip, 0);
      usb_low_SetCMTLoopCount (chip, StepCount / 2);
    }
  usb_low_EnableMotor (chip, SANE_TRUE);
  return SANE_TRUE;

}

SANE_Bool
usb_mid_motor1200_PrepareHome (ma1017 * chip)
{
  usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE);
  /* Make it in 600dpi */
  usb_low_SetIO3 (chip, SANE_TRUE);	/* (IO3) ? High power : Low power */
  usb_low_MoveMotorHome (chip, SANE_TRUE, SANE_TRUE);
  return SANE_TRUE;
}

SANE_Bool
usb_mid_motor1200_PrepareAdjust (ma1017 * chip, CHANNEL chSelect)
{
  usb_low_SetCMTTable (chip, 0, chSelect, SANE_FALSE, SANE_TRUE);
  usb_low_SetCMTTable (chip, 1, chSelect, SANE_FALSE, SANE_TRUE);
  usb_low_SetCMTTable (chip, 2, chSelect, SANE_FALSE, SANE_FALSE);
  usb_low_SetCMTTableLength (chip, 2);
  usb_low_SetCMTSecondPosition (chip, 0);
  usb_low_SetCMTLoopCount (chip, 0xefff);
  return SANE_TRUE;
}


/* adafrontend -------------------------------------------------------------*/

#if 0
static SANE_Byte s_PGATable[PGATableSize] = 
{
  0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 0x10, 0x90, 0x50, 0xd0,
  0x30, 0xb0, 0x70, 0xf0, 0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
  0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, 0x04, 0x84, 0x44, 0xc4,
  0x24, 0xa4, 0x64, 0xe4, 0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
  0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 0x1c, 0x9c, 0x5c, 0xdc,
  0x3c, 0xbc, 0x7c, 0xfc
};
#endif

void
usb_mid_front_SetFrontEndMode (ma1017 * chip, SANE_Byte Mode)
{
  usb_low_SetSerialFormat (chip, Mode);
}

void
usb_mid_front_Enable (ma1017 * chip, SANE_Bool isEnable)
{
  usb_low_TurnFrontEndMode (chip, isEnable);
}

void
usb_mid_front_SetTopReference (ma1017 * chip, SANE_Byte Top)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x00);
  usb_low_SetSerialByte2 (chip, Top);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetRedOffset (ma1017 * chip, SANE_Byte Offset)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x10);
  usb_low_SetSerialByte2 (chip, Offset);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetGreenOffset (ma1017 * chip, SANE_Byte Offset)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x50);
  usb_low_SetSerialByte2 (chip, Offset);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetBlueOffset (ma1017 * chip, SANE_Byte Offset)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x30);
  usb_low_SetSerialByte2 (chip, Offset);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetRedPGA (ma1017 * chip, SANE_Byte PGA)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x40);
  usb_low_SetSerialByte2 (chip, PGA);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetGreenPGA (ma1017 * chip, SANE_Byte PGA)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x20);
  usb_low_SetSerialByte2 (chip, PGA);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetBluePGA (ma1017 * chip, SANE_Byte PGA)
{
  usb_mid_front_Enable (chip, SANE_TRUE);
  usb_low_SetSerialByte1 (chip, 0x60);
  usb_low_SetSerialByte2 (chip, PGA);
  usb_mid_front_Enable (chip, SANE_FALSE);
}

void
usb_mid_front_SetRGBSignal (ma1017 * chip)
{
  usb_low_SetRedRef (chip, 0xEF);
  usb_low_SetGreenRef (chip, 0xF7);
  usb_low_SetBlueRef (chip, 0xFF);
}

#if 0
int
usb_mid_front_MaxPGAIndex ()
{
  return PGATableSize - 1;
}
#endif
